@@ -126,21 +126,21 @@ class GatedStateSpacesLM(nn.Module):
 
         self.layers = nn.ModuleList([])
         for _ in range(depth):
-            self.layers.append(nn.ModuleList([
+            self.layers.append(
                 GSS(
                     dim = dim,
                     dss_kernel_H = dss_kernel_H,
                     dss_kernel_N = dss_kernel_N,
                     dim_expansion_factor = dim_expansion_factor
                 )
-            ]))
+            )
 
         self.to_logits = nn.Linear(dim, num_tokens, bias = False)
 
     def forward(self, x, labels = None):
         x = self.token_emb(x)
 
-        for gss, in self.layers:
+        for gss in self.layers:
             x = gss(x)
 
         logits = self.to_logits(x)
